@@ -77,11 +77,7 @@ impl CubicCoeff {
 
 // TODO: to a custom type?
 pub fn new_t_values() -> [NormalizedF32Exclusive; 3] {
-    [
-        NormalizedF32Exclusive::ANY,
-        NormalizedF32Exclusive::ANY,
-        NormalizedF32Exclusive::ANY,
-    ]
+    [NormalizedF32Exclusive::ANY; 3]
 }
 
 pub fn chop_quad_at(src: &[Point], t: NormalizedF32Exclusive, dst: &mut [Point; 5]) {
@@ -183,6 +179,16 @@ pub fn chop_cubic_at2(src: &[Point; 4], t: NormalizedF32Exclusive, dst: &mut [Po
     dst[6] = Point::from_f32x2(p3);
 }
 
+// Quad'(t) = At + B, where
+// A = 2(a - 2b + c)
+// B = 2(b - a)
+// Solve for t, only if it fits between 0 < t < 1
+pub(crate) fn find_quad_extrema(a: f32, b: f32, c: f32) -> Option<NormalizedF32Exclusive> {
+    // At + B == 0
+    // t = -B / A
+    valid_unit_divide(a - b, a - b - b + c)
+}
+
 pub fn valid_unit_divide(mut numer: f32, mut denom: f32) -> Option<NormalizedF32Exclusive> {
     if numer < 0.0 {
         numer = -numer;
@@ -310,7 +316,7 @@ fn formulate_f1_dot_f2(src: &[f32; 4]) -> [f32; 4] {
     [c * c, 3.0 * b * c, 2.0 * b * b + c * a, a * b]
 }
 
-/// Solve coeff(t) == 0, returning the number of roots that lie withing 0 < t < 1.
+/// Solve coeff(t) == 0, returning the number of roots that lie within 0 < t < 1.
 /// coeff[0]t^3 + coeff[1]t^2 + coeff[2]t + coeff[3]
 ///
 /// Eliminates repeated roots (so that all t_values are distinct, and are always
@@ -449,6 +455,26 @@ fn eval_cubic_derivative(src: &[Point; 4], t: NormalizedF32) -> Point {
     Point::from_f32x2(coeff.eval(f32x2::splat(t.get())))
 }
 
+// Cubic'(t) = At^2 + Bt + C, where
+// A = 3(-a + 3(b - c) + d)
+// B = 6(a - 2b + c)
+// C = 3(b - a)
+// Solve for t, keeping only those that fit between 0 < t < 1
+pub(crate) fn find_cubic_extrema(
+    a: f32,
+    b: f32,
+    c: f32,
+    d: f32,
+    t_values: &mut [NormalizedF32Exclusive; 3],
+) -> usize {
+    // we divide A,B,C by 3 to simplify
+    let aa = d - a + 3.0 * (b - c);
+    let bb = 2.0 * (a - b - b + c);
+    let cc = b - a;
+
+    find_unit_quad_roots(aa, bb, cc, t_values)
+}
+
 // http://www.faculty.idc.ac.il/arik/quality/appendixA.html
 //
 // Inflection means that curvature is zero.
@@ -736,7 +762,7 @@ impl Conic {
             dst[i] = Conic::from_points(&quadrant_points[i * 2..], QUADRANT_WEIGHT);
         }
 
-        // Now compute any remaing (sub-90-degree) arc for the last conic
+        // Now compute any remaining (sub-90-degree) arc for the last conic
         let final_pt = Point::from_xy(x, y);
         let last_q = quadrant_points[quadrant * 2]; // will already be a unit-vector
         let dot = last_q.dot(final_pt);
